package com.xkrs.model.entity;

import javax.persistence.*;
import java.io.Serializable;

@Entity
@Table(name = "sys_user")
public class SysUserEntity implements Serializable {

    /**
     * 指定主键，建立自增序列，主键值取自序列
     */
    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "sys_user_seq_gen")
    @SequenceGenerator(name = "sys_user_seq_gen", sequenceName = "sys_user_id_seq", allocationSize = 1)
    private Long id;

    /**
     * 用户名
     */
    @Column(nullable = false, unique = true)
    private String userName;

    /**
     * 用户真实姓名
     */
    private String reallyName;

    /**
     * 密码
     */
    @Column(nullable = false)
    private String password;

    @Column(nullable = false)
    private String salt;

    @Column(unique = true)
    private String telephone;

    private String countyCode;

    private String countyName;

    /**
     * 0：正常用户
     * 1：被禁用的用户
     */
    @Column(nullable = false)
    private Integer activeFlag;

    @Column(nullable = false)
    private String addTime;

    /**
     * 账号类别
     */
    private String accountType;

    /**
     * 登录次数
     */
    private Integer loginNum;

    /**
     * 用户最新登录时间
     */
    private String loginLastTime;

    /**
     * 注册时的UA
     */
    @Column(columnDefinition = "varchar(1024)")
    private String userAgent;

    /**
     * vip等级
     */
    private Integer vipLevel;

    /**
     * 是否接收短信
     */
    private Integer receiveSms;

    /**
     * 一共接收了多少条短信
     */
    private Long receiveSmsCount;

    /**
     * 备注
     */
    private String remark;

    /**
     * 推送ID
     */
    private String pushId;

    /**
     * VIP时间范围列表Json
     */
    @Column(columnDefinition = "varchar(40960)")
    private String vipTimeRangeJson;

    public SysUserEntity() {
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public String getReallyName() {
        return reallyName;
    }

    public void setReallyName(String reallyName) {
        this.reallyName = reallyName;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getSalt() {
        return salt;
    }

    public void setSalt(String salt) {
        this.salt = salt;
    }

    public String getTelephone() {
        return telephone;
    }

    public void setTelephone(String telephone) {
        this.telephone = telephone;
    }

    public String getCountyCode() {
        return countyCode;
    }

    public void setCountyCode(String countyCode) {
        this.countyCode = countyCode;
    }

    public String getCountyName() {
        return countyName;
    }

    public void setCountyName(String countyName) {
        this.countyName = countyName;
    }

    public Integer getActiveFlag() {
        return activeFlag;
    }

    public void setActiveFlag(Integer activeFlag) {
        this.activeFlag = activeFlag;
    }

    public String getAddTime() {
        return addTime;
    }

    public void setAddTime(String addTime) {
        this.addTime = addTime;
    }

    public String getAccountType() {
        return accountType;
    }

    public void setAccountType(String accountType) {
        this.accountType = accountType;
    }

    public Integer getLoginNum() {
        return loginNum;
    }

    public void setLoginNum(Integer loginNum) {
        this.loginNum = loginNum;
    }

    public String getLoginLastTime() {
        return loginLastTime;
    }

    public void setLoginLastTime(String loginLastTime) {
        this.loginLastTime = loginLastTime;
    }

    public String getUserAgent() {
        return userAgent;
    }

    public void setUserAgent(String userAgent) {
        this.userAgent = userAgent;
    }

    public Integer getVipLevel() {
        return vipLevel;
    }

    public void setVipLevel(Integer vipLevel) {
        this.vipLevel = vipLevel;
    }

    public Integer getReceiveSms() {
        return receiveSms;
    }

    public void setReceiveSms(Integer receiveSms) {
        this.receiveSms = receiveSms;
    }

    public Long getReceiveSmsCount() {
        return receiveSmsCount;
    }

    public void setReceiveSmsCount(Long receiveSmsCount) {
        this.receiveSmsCount = receiveSmsCount;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public String getPushId() {
        return pushId;
    }

    public void setPushId(String pushId) {
        this.pushId = pushId;
    }

    public String getVipTimeRangeJson() {
        return vipTimeRangeJson;
    }

    public void setVipTimeRangeJson(String vipTimeRangeJson) {
        this.vipTimeRangeJson = vipTimeRangeJson;
    }

    @Override
    public String toString() {
        return "SysUserEntity{" +
                "id=" + id +
                ", userName='" + userName + '\'' +
                ", reallyName='" + reallyName + '\'' +
                ", password='" + password + '\'' +
                ", salt='" + salt + '\'' +
                ", telephone='" + telephone + '\'' +
                ", countyCode='" + countyCode + '\'' +
                ", countyName='" + countyName + '\'' +
                ", activeFlag=" + activeFlag +
                ", addTime='" + addTime + '\'' +
                ", accountType='" + accountType + '\'' +
                ", loginNum=" + loginNum +
                ", loginLastTime='" + loginLastTime + '\'' +
                ", userAgent='" + userAgent + '\'' +
                ", vipLevel=" + vipLevel +
                ", receiveSms=" + receiveSms +
                ", receiveSmsCount=" + receiveSmsCount +
                ", remark='" + remark + '\'' +
                ", pushId='" + pushId + '\'' +
                ", vipTimeRangeJson='" + vipTimeRangeJson + '\'' +
                '}';
    }
}