package com.xkrs.service;

import com.xkrs.model.entity.SysUserEntity;
import com.xkrs.model.qo.SysUserQo;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * 系统用户服务接口
 */
public interface SysUserService {

    /**
     * 检查用户名
     *
     * @param userName
     * @return
     */
    boolean checkUserName(String userName);

    /**
     * 新增用户
     *
     * @param sysUserQo
     * @return
     */
    String addUser(SysUserQo sysUserQo, HttpServletRequest servletRequest);

    /**
     * 新增用户（直接注册版）
     *
     * @param sysUserQo
     * @return
     */
    String addUser2(SysUserQo sysUserQo);

    /**
     * 删除系统用户
     *
     * @param userName
     * @return
     */
    String deleteSysUser(String userName);

    /**
     * 获取系统用户实体根据用户名
     *
     * @param userName
     * @return
     */
    SysUserEntity getSysUserByUserName(String userName);

    /**
     * 系统用户更新
     *
     * @param sysUserQo
     * @return
     */
    int updateSysUser(SysUserQo sysUserQo);

    /**
     * 查询用户的信息
     *
     * @return
     */
    List<SysUserEntity> selectAllUser();

    /**
     * 启用
     *
     * @param userId
     */
    void updateEnable(Long userId);

    /**
     * 禁用
     *
     * @param userId
     */
    void updateDisable(Long userId);

    /**
     * 用户修改密码
     *
     * @param oldPassword
     * @param newPassword
     * @param confirmPassword
     * @param sysUserEntity
     * @return
     */
    String updatePassword(String oldPassword, String newPassword, String confirmPassword, SysUserEntity sysUserEntity);

    /**
     * 管理员修改用户的密码
     *
     * @param userId
     * @param newPassword
     * @param confirmPassword
     * @return
     */
    String adminUpdatePassword(Long userId, String newPassword, String confirmPassword);

    /**
     * 获取系统用户实体根据用户名
     *
     * @param userName
     * @return
     */
    SysUserEntity getUserByUserName(String userName);

    /**
     * 记录用户登录次数
     *
     * @param userId
     * @param loginNum
     */
    void updateLoginNum(Long userId, Integer loginNum);

    /**
     * 记录用户登录时间
     *
     * @param userId
     * @param loginLastTime
     */
    void updateLoginLastTime(Long userId, String loginLastTime);

}
